@@ -11,7 +11,7 @@ __PACKAGE__->mk_ro_accessors(
     qw(label value uom warning critical min max)
 );
 
-use Nagios::Plugin::Functions;
+use Nagios::Plugin::Functions qw($value_re);
 use Nagios::Plugin::Threshold;
 use Nagios::Plugin::Range;
 our ($VERSION) = $Nagios::Plugin::Functions::VERSION;
@@ -22,12 +22,11 @@ sub import {
 	Nagios::Plugin::Functions::_use_die($_);
 }
 
-my $value_re = qr/[-+]?[\d\.]+/;
-my $value_re_with_negative_infinity = qr/$value_re|~/;
+my $value_with_negative_infinity = qr/$value_re|~/;
 sub _parse {
 	my $class = shift;
 	my $string = shift;
-	$string =~ s/^([^=]+)=($value_re)([\w%]*);?($value_re_with_negative_infinity\:?$value_re?)?;?($value_re_with_negative_infinity\:?$value_re?)?;?($value_re)?;?($value_re)?\s*//o;
+	$string =~ s/^([^=]+)=($value_re)([\w%]*);?($value_with_negative_infinity\:?$value_re?)?;?($value_with_negative_infinity\:?$value_re?)?;?($value_re)?;?($value_re)?\s*//o;
 	return undef unless ((defined $1 && $1 ne "") && (defined $2 && $2 ne ""));
     my $p = $class->new(
         label => $1, value => $2+0, uom => $3, warning => $4, critical => $5, 
@@ -70,19 +69,23 @@ sub parse_perfstring {
 }
 
 sub rrdlabel {
+	my $self = shift;
+	my $name = $self->clean_label;
+	# Shorten
+	return substr( $name, 0, 19 );
+}
+
+sub clean_label {
 	my $self = shift;
 	my $name = $self->label;
 	if ($name eq "/") {
 		$name = "root";
-    }
-	# If filesystem name, remove initial / and convert subsequent "/" to "_"
-	elsif ($name =~ s/^\///) {
+	} elsif ( $name =~ s/^\/// ) {
 		$name =~ s/\//_/g;
 	}
-	# Convert bad chars
+	# Convert all other characters
 	$name =~ s/\W/_/g;
-	# Shorten
-	return substr( $name, 0, 19 );
+	return $name;
 }
 
 # Backward compatibility: create a threshold object on the fly as requested
@@ -213,9 +216,18 @@ Returns a string based on 'label' that is suitable for use as dataset name of
 an RRD i.e. munges label to be 1-19 characters long with only characters 
 [a-zA-Z0-9_].
 
+This calls $self->clean_label and then truncates to 19 characters.
+
 There is no guarantee that multiple N:P:Performance objects will have unique 
 rrdlabels.
 
+=item clean_label
+
+Returns a "clean" label for use as a dataset name in RRD, ie, it converts
+characters that are not [a-zA-Z0-9_] to _.
+
+It also converts "/" to "root" and "/{name}" to "{name}".
+
 =item perfoutput
 
 Outputs the data in Nagios::Plugin perfdata format i.e. 
